/*****************************************************************************
 * Copyright (C) 2009-2013   this file is part of the NPTool Project         *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: N. de Sereville  contact address: deserevi@ipno.in2p3.fr *
 *                                                                           *
 * Creation Date  : 12/01/11                                                 *
 * Last update    :                                                          *
 *---------------------------------------------------------------------------*
 * Decription: Define the W1 detector from Micron                            *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/

// C++ headers
#include <sstream>
#include <string>
#include <cmath>

// G4 Geometry headers
#include "G4Box.hh"
#include "G4Tubs.hh"

// G4 various headers
#include "G4Material.hh"
#include "G4VisAttributes.hh"
#include "G4Colour.hh"
#include "G4RotationMatrix.hh"
#include "G4Transform3D.hh"
#include "G4PVPlacement.hh"
#include "G4PVDivision.hh"

// G4 sensitive
#include "G4SDManager.hh"
#include "G4MultiFunctionalDetector.hh"

// NPTool headers
#include "ObsoleteGeneralScorers.hh"
#include "MaterialManager.hh"
#include "W1.hh"
#include "W1Scorers.hh"
#include "TW1Data.h"
#include "RootOutput.h"

// CLHEP
#include "CLHEP/Random/RandGauss.h"

using namespace std;
using namespace CLHEP;
using namespace W1SQUARE;



W1::W1()
   : m_Event(new TW1Data)
{
   InitializeMaterials();
}



W1::~W1()
{
   delete m_Event;
   delete m_MaterialAluminium;
   delete m_MaterialSilicon;
   delete m_MaterialIron;
   delete m_MaterialVacuum ;
}



void W1::AddDetector(G4ThreeVector X1_Y1, G4ThreeVector X16_Y1,
                     G4ThreeVector X1_Y16, G4ThreeVector X16_Y16)
{
   m_DefinitionType.push_back(true);

   m_X1_Y1.push_back(X1_Y1);
   m_X16_Y1.push_back(X16_Y1);
   m_X1_Y16.push_back(X1_Y16);
   m_X16_Y16.push_back(X16_Y16);

   m_R.push_back(0);
   m_Theta.push_back(0);
   m_Phi.push_back(0);
   m_beta_u.push_back(0);
   m_beta_v.push_back(0);
   m_beta_w.push_back(0);
}



void W1::AddDetector(G4double R, G4double Theta, G4double Phi,
                     G4double beta_u, G4double beta_v, G4double beta_w)
{
  G4ThreeVector empty = G4ThreeVector(0, 0, 0);

  m_DefinitionType.push_back(false);

  m_R.push_back(R);
  m_Theta.push_back(Theta);
  m_Phi.push_back(Phi);
  m_beta_u.push_back(beta_u);
  m_beta_v.push_back(beta_v);
  m_beta_w.push_back(beta_w);

  m_X1_Y1.push_back(empty);
  m_X16_Y1.push_back(empty);
  m_X1_Y16.push_back(empty);
  m_X16_Y16.push_back(empty);

}



void W1::VolumeMaker(G4int             DetecNumber,
                     G4ThreeVector     position,
                     G4RotationMatrix* rotation,
                     G4LogicalVolume*  world)
{
   G4double NbrTelescopes = DetecNumber;
   G4String DetectorNumber;
   ostringstream Number;
   Number << NbrTelescopes;
   DetectorNumber = Number.str();

   ////////////////////////////////////////////////////////////////
   ////////////// Starting Volume Definition //////////////////////
   ////////////////////////////////////////////////////////////////
   G4String Name = "W1Square" + DetectorNumber;

   // Definition of the volume containing the sensitive detector
   G4Box*           solidW1 = new G4Box(Name, 0.5*FaceFront, 0.5*FaceFront, 0.5*Length);
   G4LogicalVolume* logicW1 = new G4LogicalVolume(solidW1, m_MaterialVacuum, Name, 0, 0, 0);

   new G4PVPlacement(G4Transform3D(*rotation, position), logicW1, Name, world, false, 0);

   logicW1->SetVisAttributes(G4VisAttributes::Invisible);
   if (m_non_sensitive_part_visiualisation) logicW1->SetVisAttributes(G4VisAttributes(G4Colour(0.90, 0.90, 0.90)));

   // Aluminium dead layers
   G4ThreeVector positionAluStripFront = G4ThreeVector(0, 0, AluStripFront_PosZ);
   G4ThreeVector positionAluStripBack  = G4ThreeVector(0, 0, AluStripBack_PosZ);

   G4Box*           solidAluStrip = new G4Box("AluBox", 0.5*SiliconFace, 0.5*SiliconFace, 0.5*AluStripThickness);
//   G4LogicalVolume* logicAluStrip = new G4LogicalVolume(solidAluStrip, m_MaterialAluminium, "logicAluStrip", 0, 0, 0);
   G4LogicalVolume* logicAluStrip = new G4LogicalVolume(solidAluStrip, m_MaterialVacuum, "logicAluStrip", 0, 0, 0);

   new G4PVPlacement(0, positionAluStripFront, logicAluStrip, Name + "_AluStripFront", logicW1, false, 0);
   new G4PVPlacement(0, positionAluStripBack,  logicAluStrip, Name + "_AluStripBack",  logicW1, false, 0);

   logicAluStrip->SetVisAttributes(G4VisAttributes::Invisible);

   // Silicon detector itself
   G4ThreeVector  positionSilicon = G4ThreeVector(0, 0, Silicon_PosZ);

   G4Box*           solidSilicon = new G4Box("solidSilicon", 0.5*SiliconFace, 0.5*SiliconFace, 0.5*SiliconThickness);
   G4LogicalVolume* logicSilicon = new G4LogicalVolume(solidSilicon, m_MaterialSilicon, "logicSilicon", 0, 0, 0);

   new G4PVPlacement(0, positionSilicon, logicSilicon, Name + "_Silicon", logicW1, false, 0);

   // Set Silicon strip sensible
   logicSilicon->SetSensitiveDetector(m_Scorer);

   ///Visualisation of Silicon Strip
   G4VisAttributes* SiliconVisAtt = new G4VisAttributes(G4Colour(0.0, 0.0, 0.9));
   logicSilicon->SetVisAttributes(SiliconVisAtt);
}



// Read stream at Configfile to pick-up parameters of detector (Position,...)
// Called in DetecorConstruction::ReadDetextorConfiguration Method
void W1::ReadConfiguration(string Path)
{
   ifstream ConfigFile;
   ConfigFile.open(Path.c_str());
   string LineBuffer, DataBuffer;

   // A:X1_Y1     --> X:1    Y:1
   // B:X16_Y1    --> X:16   Y:1
   // C:X1_Y16    --> X:1    Y:16
   // D:X16_Y16   --> X:16   Y:16

   G4double Ax, Bx, Cx, Dx, Ay, By, Cy, Dy, Az, Bz, Cz, Dz;
   G4ThreeVector A, B, C, D;
   G4double Theta = 0, Phi = 0, R = 0, beta_u = 0, beta_v = 0, beta_w = 0;

   bool ReadingStatus = false;

   bool check_A = false;
   bool check_C = false;
   bool check_B = false;
   bool check_D = false;
   bool check_R     = false;
   bool check_Theta = false;
   bool check_Phi   = false;
   bool checkVis = false ;

   while (!ConfigFile.eof()) {
      getline(ConfigFile, LineBuffer);

      if (LineBuffer.compare(0, 2, "W1") == 0) {
         G4cout << "///" << G4endl;
         G4cout << "W1 element found: " << G4endl;
         ReadingStatus = true;
      }
      else ReadingStatus = false;

      while (ReadingStatus) {
         ConfigFile >> DataBuffer;

         // Search for comment Symbol %
         if (DataBuffer.compare(0, 1, "%") == 0) {
            ConfigFile.ignore ( std::numeric_limits<std::streamsize>::max(), '\n' );
         }

         // Position method
         else if (DataBuffer.compare(0, 6, "X1_Y1=") == 0) {
            check_A = true;
            ConfigFile >> DataBuffer;
            Ax = atof(DataBuffer.c_str());
            Ax = Ax * mm;
            ConfigFile >> DataBuffer;
            Ay = atof(DataBuffer.c_str());
            Ay = Ay * mm;
            ConfigFile >> DataBuffer;
            Az = atof(DataBuffer.c_str());
            Az = Az * mm;

            A = G4ThreeVector(Ax, Ay, Az);
            cout << "X1 Y1 corner position : " << A << endl;
         }

         else if (DataBuffer.compare(0, 7, "X16_Y1=") == 0) {
            check_B = true;
            ConfigFile >> DataBuffer;
            Bx = atof(DataBuffer.c_str());
            Bx = Bx * mm;
            ConfigFile >> DataBuffer;
            By = atof(DataBuffer.c_str());
            By = By * mm;
            ConfigFile >> DataBuffer;
            Bz = atof(DataBuffer.c_str());
            Bz = Bz * mm ;

            B = G4ThreeVector(Bx, By, Bz);
            cout << "X16 Y1 corner position : " << B << endl;
         }

         else if (DataBuffer.compare(0, 7, "X1_Y16=") == 0) {
            check_C = true;
            ConfigFile >> DataBuffer;
            Cx = atof(DataBuffer.c_str());
            Cx = Cx * mm;
            ConfigFile >> DataBuffer;
            Cy = atof(DataBuffer.c_str());
            Cy = Cy * mm;
            ConfigFile >> DataBuffer;
            Cz = atof(DataBuffer.c_str());
            Cz = Cz * mm;

            C = G4ThreeVector(Cx, Cy, Cz);
            cout << "X1 Y16 corner position : " << C << endl;
         }

         else if (DataBuffer.compare(0, 8, "X16_Y16=") == 0) {
            check_D = true;
            ConfigFile >> DataBuffer; 
            Dx = atof(DataBuffer.c_str());
            Dx = Dx * mm;
            ConfigFile >> DataBuffer;
            Dy = atof(DataBuffer.c_str());
            Dy = Dy * mm;
            ConfigFile >> DataBuffer;
            Dz = atof(DataBuffer.c_str());
            Dz = Dz * mm;

            D = G4ThreeVector(Dx, Dy, Dz);
            cout << "X16 Y16 corner position : " << D << endl;
         }


         // Angle method
         else if (DataBuffer.compare(0, 6, "THETA=") == 0) {
            check_Theta = true;
            ConfigFile >> DataBuffer ;
            Theta = atof(DataBuffer.c_str()) ;
            Theta = Theta * deg;
            cout << "Theta:  " << Theta / deg << endl;
         }

         else if (DataBuffer.compare(0, 4, "PHI=") == 0) {
            check_Phi = true;
            ConfigFile >> DataBuffer ;
            Phi = atof(DataBuffer.c_str()) ;
            Phi = Phi * deg;
            cout << "Phi:  " << Phi / deg << endl;
         }

         else if (DataBuffer.compare(0, 2, "R=") == 0) {
            check_R = true;
            ConfigFile >> DataBuffer ;
            R = atof(DataBuffer.c_str()) ;
            R = R * mm;
            cout << "R:  " << R / mm << endl;
         }

         else if (DataBuffer.compare(0, 5, "BETA=") == 0) {
            ConfigFile >> DataBuffer;
            beta_u = atof(DataBuffer.c_str());
            beta_u = beta_u * deg;
            ConfigFile >> DataBuffer;
            beta_v = atof(DataBuffer.c_str());
            beta_v = beta_v * deg;
            ConfigFile >> DataBuffer;
            beta_w = atof(DataBuffer.c_str());
            beta_w = beta_w * deg;
            G4cout << "Beta:  " << beta_u / deg << " " << beta_v / deg << " " << beta_w / deg << G4endl  ;
         }

         else if (DataBuffer.compare(0, 4, "VIS=") == 0) {
            checkVis = true;
            ConfigFile >> DataBuffer;
            if (DataBuffer.compare(0, 3, "all") == 0) m_non_sensitive_part_visiualisation = true;
         }

         else {
            ///////////////////////////////////////////////////
            // If no Detector Token and no comment, toggle out
            ReadingStatus = false;
            G4cout << "Wrong Token Sequence: Getting out " << DataBuffer << G4endl;
         }

         // Add The previously define detector
         // With position method
         if ((check_A && check_B && check_C && check_D && checkVis) && !(check_Theta && check_Phi && check_R)) {
            ReadingStatus = false;
            check_A  = false;
            check_C  = false;
            check_B  = false;
            check_D  = false;
            checkVis = false;

            AddDetector(A, B, C, D);
         }

         // with angle method
         if ((check_Theta && check_Phi && check_R && checkVis) && !(check_A && check_B && check_C && check_D)) {
            ReadingStatus = false;
            check_R     = false;
            check_Theta = false;
            check_Phi   = false;
            checkVis    = false;

            AddDetector(R, Theta, Phi, beta_u, beta_v, beta_w);
         }
      }
   }
}



// Construct detector and inialise sensitive part.
// Called After DetecorConstruction::AddDetector Method
void W1::ConstructDetector(G4LogicalVolume* world)
{
   G4RotationMatrix* W1rot    = NULL;
   G4ThreeVector     W1pos    = G4ThreeVector(0, 0, 0) ;
   G4ThreeVector     W1u      = G4ThreeVector(0, 0, 0) ;
   G4ThreeVector     W1v      = G4ThreeVector(0, 0, 0) ;
   G4ThreeVector     W1w      = G4ThreeVector(0, 0, 0) ;
   G4ThreeVector     W1Center = G4ThreeVector(0, 0, 0) ;
   
   G4int NumberOfDetector = m_DefinitionType.size() ;
   for (G4int i = 0; i < NumberOfDetector; i++) {
      // By Point
      if (m_DefinitionType[i]) {
         // (u,v,w) unitary vector associated to telescope referencial
         // (u,v) // to silicon plan
         // w perpendicular to (u,v) plan and pointing ThirdStage
         W1u = m_X16_Y1[i] - m_X1_Y1[i];
         W1u = W1u.unit();

         W1v = m_X1_Y16[i] - m_X1_Y1[i];
         W1v = W1v.unit();

         W1w = W1u.cross(W1v);
         W1w = W1w.unit();

         W1Center = (m_X1_Y1[i] + m_X1_Y16[i] + m_X16_Y1[i] + m_X16_Y16[i]) / 4;

         // Passage Matrix from Lab Referential to Telescope Referential
         W1rot = new G4RotationMatrix(W1u, W1v, W1w);
         // translation to place Telescope
         W1pos = W1w * Length * 0.5 + W1Center;
      }

      // By Angle
      else {
         G4double Theta = m_Theta[i];
         G4double Phi   = m_Phi[i];

         // (u,v,w) unitary vector associated to telescope referencial
         // (u,v) // to silicon plan
         // w perpendicular to (u,v) plan and pointing ThirdStage
         // Phi is angle between X axis and projection in (X,Y) plan
         // Theta is angle between  position vector and z axis
         G4double wX = m_R[i] * sin(Theta / rad) * cos(Phi / rad);
         G4double wY = m_R[i] * sin(Theta / rad) * sin(Phi / rad);
         G4double wZ = m_R[i] * cos(Theta / rad);
         W1w = G4ThreeVector(wX, wY, wZ);

         // vector corresponding to the center of the module
         W1Center = W1w;

         // vector parallel to one axis of silicon plane
         G4double ii = cos(Theta / rad) * cos(Phi / rad);
         G4double jj = cos(Theta / rad) * sin(Phi / rad);
         G4double kk = -sin(Theta / rad);
         G4ThreeVector Y = G4ThreeVector(ii, jj, kk);

         W1w = W1w.unit();
         W1u = W1w.cross(Y);
         W1v = W1w.cross(W1u);
         W1v = W1v.unit();
         W1u = W1u.unit();

         // Passage Matrix from Lab Referential to Telescope Referential
         // MUST2
         W1rot = new G4RotationMatrix(W1u, W1v, W1w);
         // Telescope is rotate of Beta angle around W1v axis.
         W1rot->rotate(m_beta_u[i], W1u);
         W1rot->rotate(m_beta_v[i], W1v);
         W1rot->rotate(m_beta_w[i], W1w);
         // translation to place Telescope
         W1pos = W1w * Length * 0.5 + W1Center;
      }

      VolumeMaker(i + 1, W1pos, W1rot, world);
   }

   delete W1rot;
}



// Connect the GaspardTrackingData class to the output TTree
// of the simulation
void W1::InitializeRootOutput()
{
   RootOutput *pAnalysis = RootOutput::getInstance();
   TTree *pTree = pAnalysis->GetTree();
   pTree->Branch("W1", "TW1Data", &m_Event);
}



// Read sensitive part and fill the Root tree.
// Called at in the EventAction::EndOfEventAvtion
void W1::ReadSensitive(const G4Event* event)
{
   // Clear ROOT objects
   m_Event->Clear();

   //////////////////////////////////////////////////////////////////////////////////
   /////////////// Variables  Used to Read Event Map of detector ////////////////////
   //////////////////////////////////////////////////////////////////////////////////
   // First Stage
   std::map<G4int, G4int*>::iterator DetectorNumber_itr;
   std::map<G4int, G4double*>::iterator Energy_itr;
   std::map<G4int, G4double*>::iterator Time_itr;
   std::map<G4int, G4double*>::iterator X_itr;
   std::map<G4int, G4double*>::iterator Y_itr;
   std::map<G4int, G4double*>::iterator Pos_X_itr;
   std::map<G4int, G4double*>::iterator Pos_Y_itr;
   std::map<G4int, G4double*>::iterator Pos_Z_itr;
   std::map<G4int, G4double*>::iterator Ang_Theta_itr;
   std::map<G4int, G4double*>::iterator Ang_Phi_itr;

   G4THitsMap<G4int>*    DetectorNumberHitMap;
   G4THitsMap<G4double>* EnergyHitMap;
   G4THitsMap<G4double>* TimeHitMap;
   G4THitsMap<G4double>* XHitMap;
   G4THitsMap<G4double>* YHitMap;
   G4THitsMap<G4double>* PosXHitMap;
   G4THitsMap<G4double>* PosYHitMap;
   G4THitsMap<G4double>* PosZHitMap;
   G4THitsMap<G4double>* AngThetaHitMap;
   G4THitsMap<G4double>* AngPhiHitMap;

   /////////////////////////////////////////////////////////////////////////////////
   ////////////////// Read scorers  ////////////////////////////////////////////////
   /////////////////////////////////////////////////////////////////////////////////
   // Read the Scorer associated to the first Stage
   //Detector Number
   G4int StripDetCollectionID = G4SDManager::GetSDMpointer()->GetCollectionID("ScorerW1/DetectorNumber");
   DetectorNumberHitMap = (G4THitsMap<G4int>*)(event->GetHCofThisEvent()->GetHC(StripDetCollectionID));
   DetectorNumber_itr =  DetectorNumberHitMap->GetMap()->begin();

   //Energy
   G4int StripEnergyCollectionID = G4SDManager::GetSDMpointer()->GetCollectionID("ScorerW1/StripEnergy")   ;
   EnergyHitMap = (G4THitsMap<G4double>*)(event->GetHCofThisEvent()->GetHC(StripEnergyCollectionID))                    ;
   Energy_itr = EnergyHitMap->GetMap()->begin()                                                          ;

   //Time of Flight
   G4int StripTimeCollectionID = G4SDManager::GetSDMpointer()->GetCollectionID("ScorerW1/StripTime")    ;
   TimeHitMap = (G4THitsMap<G4double>*)(event->GetHCofThisEvent()->GetHC(StripTimeCollectionID))                        ;
   Time_itr = TimeHitMap->GetMap()->begin()                                                              ;

   //Strip Number X
   G4int StripXCollectionID = G4SDManager::GetSDMpointer()->GetCollectionID("ScorerW1/FrontStripNumber");
   XHitMap = (G4THitsMap<G4double>*)(event->GetHCofThisEvent()->GetHC(StripXCollectionID));
   X_itr = XHitMap->GetMap()->begin();

   //Strip Number Y
   G4int StripYCollectionID = G4SDManager::GetSDMpointer()->GetCollectionID("ScorerW1/BackStripNumber");
   YHitMap = (G4THitsMap<G4double>*)(event->GetHCofThisEvent()->GetHC(StripYCollectionID))                              ;
   Y_itr = YHitMap->GetMap()->begin()                                                                    ;

   //Interaction Coordinate X
   G4int InterCoordXCollectionID = G4SDManager::GetSDMpointer()->GetCollectionID("ScorerW1/InterCoordX")    ;
   PosXHitMap = (G4THitsMap<G4double>*)(event->GetHCofThisEvent()->GetHC(InterCoordXCollectionID))                              ;
   Pos_X_itr = PosXHitMap->GetMap()->begin()                                                                    ;

   //Interaction Coordinate Y
   G4int InterCoordYCollectionID = G4SDManager::GetSDMpointer()->GetCollectionID("ScorerW1/InterCoordY")    ;
   PosYHitMap = (G4THitsMap<G4double>*)(event->GetHCofThisEvent()->GetHC(InterCoordYCollectionID))                              ;
   Pos_Y_itr = PosYHitMap->GetMap()->begin()                                                                    ;

   //Interaction Coordinate Z
   G4int InterCoordZCollectionID = G4SDManager::GetSDMpointer()->GetCollectionID("ScorerW1/InterCoordZ")    ;
   PosZHitMap = (G4THitsMap<G4double>*)(event->GetHCofThisEvent()->GetHC(InterCoordZCollectionID))                              ;
   Pos_Z_itr = PosXHitMap->GetMap()->begin()                                                                    ;

   //Interaction Coordinate Angle Theta
   G4int InterCoordAngThetaCollectionID = G4SDManager::GetSDMpointer()->GetCollectionID("ScorerW1/InterCoordAngTheta")    ;
   AngThetaHitMap = (G4THitsMap<G4double>*)(event->GetHCofThisEvent()->GetHC(InterCoordAngThetaCollectionID))                              ;
   Ang_Theta_itr = AngThetaHitMap->GetMap()->begin()                                                                    ;

   //Interaction Coordinate Angle Phi
   G4int InterCoordAngPhiCollectionID = G4SDManager::GetSDMpointer()->GetCollectionID("ScorerW1/InterCoordAngPhi");
   AngPhiHitMap = (G4THitsMap<G4double>*)(event->GetHCofThisEvent()->GetHC(InterCoordAngPhiCollectionID));
   Ang_Phi_itr = AngPhiHitMap->GetMap()->begin();

   // Check the size of different map
   G4int sizeN = DetectorNumberHitMap->entries();
   G4int sizeE = EnergyHitMap->entries();
   G4int sizeT = TimeHitMap->entries();
   G4int sizeX = XHitMap->entries();
   G4int sizeY = YHitMap->entries();

   if (sizeE != sizeT || sizeT != sizeX || sizeX != sizeY) {
      G4cout << "No match size Si Event Map: sE:"
      << sizeE << " sT:" << sizeT << " sX:" << sizeX << " sY:" << sizeY << endl ;
      return;
   }

   // Loop on W1 number 
   for (G4int l = 0; l < sizeN; l++) {
      G4double N     = *(DetectorNumber_itr->second);
      G4int NTrackID =   DetectorNumber_itr->first - N;

      if (N > 0) {
         // Fill detector number
         m_Event->SetW1FrontEDetectorNbr(N);
         m_Event->SetW1FrontTDetectorNbr(N);
         m_Event->SetW1BackEDetectorNbr(N);
         m_Event->SetW1BackTDetectorNbr(N);

         //  Energy
         Energy_itr = EnergyHitMap->GetMap()->begin();
         for (G4int h = 0 ; h < sizeE ; h++) {
            G4int ETrackID =   Energy_itr->first - N;
            G4double E     = *(Energy_itr->second);
            if (ETrackID == NTrackID) {
               m_Event->SetW1FrontEEnergy(RandGauss::shoot(E, EnergyResolution));
               m_Event->SetW1BackEEnergy(RandGauss::shoot(E, EnergyResolution));
            }
            Energy_itr++;
         }

         //  Time
         Time_itr = TimeHitMap->GetMap()->begin();
         for (G4int h = 0 ; h < sizeT ; h++) {
            G4int TTrackID =   Time_itr->first - N;
            G4double T     = *(Time_itr->second);
            if (TTrackID == NTrackID) {
               m_Event->SetW1FrontTTime(RandGauss::shoot(T, TimeResolution));
               m_Event->SetW1BackTTime(RandGauss::shoot(T, TimeResolution));
            }
            Time_itr++;
         }

         // strip front 
         X_itr = XHitMap->GetMap()->begin();
         for (G4int h = 0 ; h < sizeX ; h++) {
            G4int XTrackID  =   X_itr->first - N;
            G4double X     = *(X_itr->second);
            if (XTrackID == NTrackID) {
               m_Event->SetW1FrontEStripNbr(X)   ;
               m_Event->SetW1FrontTStripNbr(X)   ;
            }
            X_itr++;
         }

         // strip back
         Y_itr = YHitMap->GetMap()->begin();
         for (G4int h = 0 ; h < sizeY ; h++) {
            G4int YTrackID  =   Y_itr->first - N;
            G4double Y     = *(Y_itr->second);
            if (YTrackID == NTrackID) {
               m_Event->SetW1BackEStripNbr(Y);
               m_Event->SetW1BackTStripNbr(Y);
            }
            Y_itr++;
         }

         // Pos X
         Pos_X_itr = PosXHitMap->GetMap()->begin();
         for (G4int h = 0; h < PosXHitMap->entries(); h++) {
            G4int PosXTrackID =   Pos_X_itr->first - N;
            G4double PosX     = *(Pos_X_itr->second);
            if (PosXTrackID == NTrackID) {
               ms_InterCoord->SetDetectedPositionX(PosX);
            }
            Pos_X_itr++;
         }

         // Pos Y
         Pos_Y_itr = PosYHitMap->GetMap()->begin();
         for (G4int h = 0; h < PosYHitMap->entries(); h++) {
            G4int PosYTrackID =   Pos_Y_itr->first - N;
            G4double PosY     = *(Pos_Y_itr->second);
            if (PosYTrackID == NTrackID) {
               ms_InterCoord->SetDetectedPositionY(PosY);
            }
            Pos_Y_itr++;
         }

         // Pos Z
         Pos_Z_itr = PosZHitMap->GetMap()->begin();
         for (G4int h = 0; h < PosZHitMap->entries(); h++) {
            G4int PosZTrackID =   Pos_Z_itr->first - N;
            G4double PosZ     = *(Pos_Z_itr->second);
            if (PosZTrackID == NTrackID) {
               ms_InterCoord->SetDetectedPositionZ(PosZ);
            }
            Pos_Z_itr++;
         }

         // Angle Theta
         Ang_Theta_itr = AngThetaHitMap->GetMap()->begin();
         for (G4int h = 0; h < AngThetaHitMap->entries(); h++) {
            G4int AngThetaTrackID =   Ang_Theta_itr->first - N;
            G4double AngTheta     = *(Ang_Theta_itr->second);
            if (AngThetaTrackID == NTrackID) {
               ms_InterCoord->SetDetectedAngleTheta(AngTheta);
            }
            Ang_Theta_itr++;
         }

         // Angle Phi
         Ang_Phi_itr = AngPhiHitMap->GetMap()->begin();
         for (G4int h = 0; h < AngPhiHitMap->entries(); h++) {
            G4int AngPhiTrackID =   Ang_Phi_itr->first - N;
            G4double AngPhi     = *(Ang_Phi_itr->second);
            if (AngPhiTrackID == NTrackID) {
               ms_InterCoord->SetDetectedAnglePhi(AngPhi);
            }
            Ang_Phi_itr++;
         }

      } // end if number of detector > 0

      DetectorNumber_itr++;
   } // end loop on detector multiplicity

   // clear map for next event
   DetectorNumberHitMap -> clear();
   EnergyHitMap         -> clear();
   TimeHitMap           -> clear();
   XHitMap              -> clear();
   YHitMap              -> clear();
   PosXHitMap           -> clear();
   PosYHitMap           -> clear();
   PosZHitMap           -> clear();
   AngThetaHitMap       -> clear();
   AngPhiHitMap         -> clear();
}



void W1::InitializeMaterials(){           
   m_MaterialSilicon = MaterialManager::getInstance()->GetMaterialFromLibrary("Si");
   m_MaterialAluminium =MaterialManager::getInstance()->GetMaterialFromLibrary("Al");
   m_MaterialIron = MaterialManager::getInstance()->GetMaterialFromLibrary("Fe");
   m_MaterialVacuum = MaterialManager::getInstance()->GetMaterialFromLibrary("Vacuum");
}



void W1::InitializeScorers(){
   // Associate Scorer
   m_Scorer = new G4MultiFunctionalDetector("ScorerW1");
   G4VPrimitiveScorer* DetNbr                           = new OBSOLETEGENERALSCORERS::PSDetectorNumber("DetectorNumber", "W1Square", 0);
   G4VPrimitiveScorer* Energy                           = new OBSOLETEGENERALSCORERS::PSEnergy("StripEnergy", "W1Square", 0);
   G4VPrimitiveScorer* TOF                              = new OBSOLETEGENERALSCORERS::PSTOF("StripTime", "W1Square", 0);
   G4VPrimitiveScorer* InteractionCoordinatesX          = new OBSOLETEGENERALSCORERS::PSInteractionCoordinatesX("InterCoordX","W1Square", 0);
   G4VPrimitiveScorer* InteractionCoordinatesY          = new OBSOLETEGENERALSCORERS::PSInteractionCoordinatesY("InterCoordY","W1Square", 0);
   G4VPrimitiveScorer* InteractionCoordinatesZ          = new OBSOLETEGENERALSCORERS::PSInteractionCoordinatesZ("InterCoordZ","W1Square", 0);
   G4VPrimitiveScorer* InteractionCoordinatesAngleTheta = new OBSOLETEGENERALSCORERS::PSInteractionCoordinatesAngleTheta("InterCoordAngTheta","W1Square", 0);
   G4VPrimitiveScorer* InteractionCoordinatesAnglePhi   = new OBSOLETEGENERALSCORERS::PSInteractionCoordinatesAnglePhi("InterCoordAngPhi","W1Square", 0);
   G4VPrimitiveScorer* ThetaStripPosition               = new W1ScorerFrontStripNumber("FrontStripNumber", 0, NbStrips);
   G4VPrimitiveScorer* PhiStripPosition                 = new W1ScorerBackStripNumber("BackStripNumber", 0, NbStrips);

   //and register it to the multifunctionnal detector
   m_Scorer->RegisterPrimitive(DetNbr);
   m_Scorer->RegisterPrimitive(Energy);
   m_Scorer->RegisterPrimitive(TOF);
   m_Scorer->RegisterPrimitive(ThetaStripPosition);
   m_Scorer->RegisterPrimitive(PhiStripPosition);
   m_Scorer->RegisterPrimitive(InteractionCoordinatesX);
   m_Scorer->RegisterPrimitive(InteractionCoordinatesY);
   m_Scorer->RegisterPrimitive(InteractionCoordinatesZ);
   m_Scorer->RegisterPrimitive(InteractionCoordinatesAngleTheta);
   m_Scorer->RegisterPrimitive(InteractionCoordinatesAnglePhi);

   //  Add All Scorer to the Global Scorer Manager
   G4SDManager::GetSDMpointer()->AddNewDetector(m_Scorer);
}
